Documentation
¶
Overview ¶
The parser package implements a parser for Lua source files. Input may be provided in a variety of forms, and the output is a parse tree representing the Lua source.
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func ParseFile ¶
ParseFile parses the source code of a single Lua file. It returns a root tree.File node representing the parsed file, any any errors that may have occurred while parsing.
The src argument may be a string, []byte, *bytes.Buffer, or io.Reader. In these cases, the filename is used only when recording positional information. If src is nil, the source is read from the file specified by filename.
Types ¶
This section is empty.
Click to show internal directories.
Click to hide internal directories.